"""
2D visualization of positions on a ScaledCanvas
"""

from casymda.visualization.entity_visualizer import EntityVisualizer


class TilemapVisualizer(EntityVisualizer):
    """ visualizes entity positions at a given canvas """
